
import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';

class S implements WidgetsLocalizations {
  const S();

  static S current;

  static const AppLocalizationDelegate delegate = AppLocalizationDelegate();

  static S of(BuildContext context) => Localizations.of<S>(context, S);

  @override
  TextDirection get textDirection => TextDirection.ltr;

  String get hello => 'World';
  String get helloWorld => 'Hello World!';
  String get firstName => 'First Name';
}

class $fr extends S {
  const $fr();

  @override
  TextDirection get textDirection => TextDirection.ltr;

  @override
  String get hello => 'World fr';
  @override
  String get helloWorld => 'Olá Mundo fr!';
  @override
  String get firstName => 'Primeiro Nome fr';

}

class $pt extends S {
  const $pt();

  @override
  TextDirection get textDirection => TextDirection.ltr;

  @override
  String get hello => 'World';
  @override
  String get helloWorld => 'Olá Mundo!';
  @override
  String get firstName => 'Primeiro Nome';

}

class $en extends S {
  const $en();

  @override
  TextDirection get textDirection => TextDirection.ltr;

  @override
  String get hello => 'World';
  @override
  String get helloWorld => 'Hello World!';
  @override
  String get firstName => 'First Name';

}

class AppLocalizationDelegate extends LocalizationsDelegate<S> {
  const AppLocalizationDelegate();

  List<Locale> get supportedLocales {
    return const <Locale>[
      Locale.fromSubtags(languageCode: 'en'),
      Locale.fromSubtags(languageCode: 'fr'),
      Locale.fromSubtags(languageCode: 'pt'),
    ];
  }

  LocaleListResolutionCallback listResolution({Locale fallback}) {
    return (List<Locale> locales, Iterable<Locale> supported) {
      if (locales == null || locales.isEmpty) {
        return fallback ?? supported.first;
      } else {
        return _resolve(locales.first, fallback, supported);
      }
    };
  }

  LocaleResolutionCallback resolution({Locale fallback}) {
    return (Locale locale, Iterable<Locale> supported) {
      return _resolve(locale, fallback, supported);
    };
  }

  @override
  Future<S> load(Locale locale) {
    final String lang = getLang(locale);
    if (lang != null) {
      switch (lang) {
        case "en":
          S.current = const $en();
          return SynchronousFuture<S>(S.current);
        case "fr":
          S.current = const $fr();
          return SynchronousFuture<S>(S.current);
        case "pt":
          S.current = const $pt();
          return SynchronousFuture<S>(S.current);
        default:
          //...
      }
    }
    return SynchronousFuture<S>(const S());
  }

  @override
  bool isSupported(Locale locale) => _isSupported(locale);

  @override
  bool shouldReload(AppLocalizationDelegate old) => false;

  /// Internal method to resolve a locale from a list of locales.
  Locale _resolve(Locale locale, Locale fallback, Iterable<Locale> supported) {
    if (locale == null || !_isSupported(locale)) {
      return fallback ?? supported.first;
    }

    final Locale languageLocale = Locale(locale.languageCode, "");
    if (supported.contains(locale)) {
      return locale;
    } else if (supported.contains(languageLocale)) {
      return languageLocale;
    } else {
      final Locale fallbackLocale = fallback ?? supported.first;
      return fallbackLocale;
    }
  }

  bool _isSupported(Locale locale) {
    if (locale != null) {
      for (Locale supportedLocale in supportedLocales) {
        if (supportedLocale.languageCode == locale.languageCode) {
          return true;
        }
      }
    }
    return false;
   }
}

String getLang(Locale locale) => locale == null
  ? null
  : locale.countryCode != null && locale.countryCode.isEmpty
    ? locale.languageCode
    : locale.toString();


